//Scilab script for analyizing imageJ EDM-SBS results.
clear
stacksize('max')

//*************************************************************************************//
///////////// Varibales to be adjusted by the user  /////////////////////////////////////
//*************************************************************************************//
Dmin=1; //Minimum diameter to be considered in the length unit (U).
Dmax=800; //Maximum diameter to be considered in the length unit (U).
Nbclasses=80; //Number of classes to consider
Reso=270/50; //Images resolution in pixels/U (exemple 5 pixels per nm), if Reso=0, the resolution is read in a file Resolution.txt that contains on each line the file name and the corresponding resolution

//The fractal analysis is performed on the basis of 
// Ü. Ö. Köylü, G. M. Faeth, T. L. Farias, and M. G. Carvalho. Fractal and projected structure properties of soot aggregates. Combustion and Flame, 100, 1995.
ka=1.095; //Factor used for the fractal analysis Np=ka(Aa/Ap).^alpha with Aa and Ap the aggregates projected surface and primary sphere projected surface
alpha=1.155;
factorDf=1; //factor used in order to link Df and kf (kf=1/f((Df+2)/Df).^(Df/2)). If f=0, kf is searched independently.
kappa=2.96; //Factor used for the determination of the particle gyration radius Rg from the maximum projected length L  L=kappa*Rg 

//Other variables
EDM_SBM_FileName="EDM-SBS.txt'; //Name of the files produced by the imageJ plugin
File_Reso="Resolution.txt";     //Name of the files containing the detail of the images resolution
ExtendedNbClasses=200;          //Number of classes for the representation of the EDM-SBS curve fittings
InitialDpgeo=50;                //Initial value to be considered for the EDM-SBS fit : geometric mean diameter
Initialsigmageo=1.5;            //Initial value to be considered for the EDM-SBS fit : geometric standard deviation

//*************************************************************************************//
///////////// Fucntions definitions//////////////////////////////////////////////////////
//*************************************************************************************//
function PlotFormatting()
  h=gca();
  h.font_size=4;
  h.font_style=2;
  h.x_label.font_size=5; 
  h.x_label.font_style=2;  
  h.y_label.font_size=5; 
  h.y_label.font_style=2;
  h=gcf();
  h.figure_size=[1024 768];
endfunction


function [S]=Sigmoid(X)
    a=0.9966; 
    b=1.9658; 
    c=-0.8515; 
    S = a./(1+exp(-(X-b)/c));
endfunction


function [S]=SDLCA(X)
    a=1.3887; 
    b=-1.0277; 
    S =0.5*(1+erf((X-a)/sqrt(2)/b));
endfunction


function [y]=fIntegration(D,Dpgeo,sigmageo)
    y=D*exp(-0.5*(log(D/Dpgeo)/log(sigmageo)).^2);
endfunction


function [S]=SSpheres(Dpourfit,Dpgeo,sigmageo)
    Nbp=size(Dpourfit,'*')
    for i=1:Nbp
        D=Dpourfit(i);
        S(i)=intg(0,D,fIntegration);
    end
    S0=intg(0,5000,fIntegration);
    S=1-S/S0;
endfunction


function err=SigmoidSearch(p,Dpourfit,Spourfit)
  Dmodal=p(1);
  ecartype=p(2);
  X=(log(Dpourfit)-log(Dmodal))/log(ecartype) ;
  Sfit = Sigmoid(X);
  err=(Spourfit-Sfit);
endfunction


function err=DLCASearch(p,Dpourfit,Spourfit)
  Dmodal=p(1);
  ecartype=p(2);
  X=(log(Dpourfit)-log(Dmodal))/log(ecartype) ;
  Sfit = SDLCA(X);
  err=(Spourfit-Sfit);
endfunction


function err=SphereSearch(p,Dpourfit,Spourfit)
  Dpgeo=p(1);
  sigmageo=p(2);
  Nbp=size(Dpourfit,'*')
  for i=1:Nbp
    Num(i)=intg(0,Dpourfit(i),fIntegration);
  end
  Den=intg(0,5000,fIntegration);
  err=Num+Den*(Spourfit-ones(Spourfit));
endfunction


function err=FractalSearch(p,X,Np)
  Df=p(1);
  kf=p(2);
  if factorDf~=0 then
    kf=1/factorDf*((Df+2)/Df).^(Df/2);
  end
  Nptheo=kf*X.^Df;
  err=(Nptheo-Np);
endfunction


//*************************************************************************************//
///////////// Reading the file containing the resolution data  ///////////////// ////////
//*************************************************************************************//


directory = uigetdir('C:\','Indicates the folder containing the results of the EDM-SBS analysis by ImageJ');
cd(directory); 
if Reso==0 then
     g=mopen(File_Reso,'r');
    [n,FileName,Resolution]=mfscanf(g,'%s\t%e\n');
    i=0;
    while (n~=-1)
        i=i+1;
        FileNameTab(i)=FileName;
        ResolutionTab(i)=Resolution;
        [n,FileName,Resolution]=mfscanf(g,'%s\t%e\n');
    end
    mclose(g);
end


//*************************************************************************************//
///////////// EDM-SBS calculation ///////////////// /////////////////////////////////////
//*************************************************************************************//

for i=1:Nbclasses
  dmin=exp((log(Dmax)-log(Dmin))/Nbclasses*(i-1)+log(Dmin));
  dmax=exp((log(Dmax)-log(Dmin))/Nbclasses*(i)+log(Dmin));
  D(i)=exp((log(dmax)-log(dmin))/Nbclasses*(i-1+0.5)+log(dmin));
end

nobj=0;
nimage=0;
Export=[];
f=mopen(EDM_SBM_FileName,'r');
[n,Type,Name,Count]=mfscanf(f,'%s\t%s\t%d\n');
while (n~=-1)
    if Reso==0 then
        NameTmp=Name;
        Select=(FileNameTab==NameTmp);
        FileNameTab(Select);
        if sum(Select)==1  then
            ResoTmp=ResolutionTab(Select);
        else
            ResoTmp=0;
        end
    else    ResoTmp=Reso;     
    end
    
    for i=1:Count
        [n,Area,Feret]=mfscanf(f,'%e\t%e\n');
        if ResoTmp~=0 then
            nobj=nobj+1;
            AaTab(nobj)=Area/ResoTmp/ResoTmp;
            RgTab(nobj)=Feret/ResoTmp/kappa;
        end
    end
    [n,Type,Name,Maxk]=mfscanf(f,'%s\t%s\t%d\n');
    nimage=nimage+1;
    NameTmp2=Name;
    DTab=[];SurfTab=[];
    for i=1:Maxk
        [n,k,Surf]=mfscanf(f,'%d\t%d\n');
        if ResoTmp~=0 then
            DTab(i)=(2*k+1)/ResoTmp;
            SurfTab(i)=Surf/ResoTmp/ResoTmp;
        end
        
    end
    if ResoTmp~=0 then
        Select=(SurfTab<0);
        if sum(Select)==0 then
            d=splin(DTab,SurfTab);
            Y=interp(D,DTab,SurfTab,d);
            Select=(D>DTab($));
            Y(Select)=0;
            NamesTab(nimage)=NameTmp2;
            Export=[Export,Y];
        else
        printf('%s\n',Name);
        end
    end
    [n,Type,Name,Count]=mfscanf(f,'%s\t%s\t%d\n');
end
mclose(f);

Ymean=mean(Export,'c');
N=sum((Export~=0),'c');
Yet=stdev(Export,'c');
S=Ymean/Ymean(1);
Select=(N>0 & D>0);
Ymean=Ymean(Select);
N=N(Select);
Yet=Yet(Select);
D=D(Select);
S=S(Select);
deltaY=2*Yet./sqrt(nimage); //Evaluation of the confidence interval at 95%
deltaS=(deltaY./Ymean)-(deltaY(1)./Ymean(1));
Selectfit=(S>=0.2) 
Spourfit=S(Selectfit);
Dpourfit=D(Selectfit);
printf('%d considered aggregates\n',nimage)
Dfit=logspace(log10(Dmin),log10(Dmax),ExtendedNbClasses)';



//*************************************************************************************//
///////////// EDM-SBS plotting ///////////////// ///////////////////
//*************************************************************************************//
h0=figure(0);
clf
set(h0,"background",8) ;//White background
plot2d(D,S,[-1],logflag='ln'); 
errbar(D,S,deltaS,deltaS)
legend(['EDM-SBS']);
xtitle('','D','S(D)/S(0)');


//*************************************************************************************//
///////////// Search of the parameters of the size distribution ///////////////////
//*************************************************************************************//
//Initial values of the modal diameter and geometric standard deviation
p0=[InitialDpgeo,Initialsigmageo]; 
xinf = [2, 1.05]; xsup = [1000000,5];
Activelegend=[%t,%f,%f,%f];

//Fit of the EDM-SBS curve by the sigmoid curve
[V,psigmoid]=leastsq(SigmoidSearch,"b",xinf,xsup,p0);
DmodSigmoid=psigmoid(1);
SigmaGeoSigmoid=psigmoid(2);
if DmodSigmoid>xinf(1) & DmodSigmoid<xsup(1) & SigmaGeoSigmoid>xinf(2) & SigmaGeoSigmoid<xsup(2)  then
    printf('Sigmoid fit for soot particles (equation 6): Geometric Diameter = %f et geometric standard deviation = %f\n',DmodSigmoid,SigmaGeoSigmoid)
    Xfit=(log(Dfit)-log(DmodSigmoid))/(log(SigmaGeoSigmoid));
    Sigmoidfit=Sigmoid(Xfit);
    plot2d(Dfit,Sigmoidfit,1,logflag='ln');
    Activelegend(2)=%t;
end


//Fit of the EDM-SBS curve by the erf function (DLCA point contact function)
[V,pDLCA]=leastsq(DLCASearch,"b",xinf,xsup,psigmoid);
DmodDLCA=pDLCA(1);
SigmaGeoDLCA=pDLCA(2);
if DmodDLCA>xinf(1) & DmodDLCA<xsup(1) & SigmaGeoDLCA>xinf(2) & SigmaGeoDLCA<xsup(2)  then
   printf('DLCA aggregates (Lognormal) Fit (equation 5-b) : Geometric Diameter = %f et geometric standard deviation = %f\n',DmodDLCA,SigmaGeoDLCA)
    Xfit=(log(Dfit)-log(DmodDLCA))/(log(SigmaGeoDLCA));
    SDLCAfit=SDLCA(Xfit);
    plot2d(Dfit,SDLCAfit,3,logflag='ln');
    Activelegend(3)=%t;
end


//Fit of the EDM-SBS curve by the function in equation 4 (Non overlapping spheres)
[V,pSpheres]=leastsq(SphereSearch,"b",xinf,xsup,[DmodDLCA,SigmaGeoDLCA]);
Dpgeo=pSpheres(1);
sigmageo=pSpheres(2);
if Dpgeo>xinf(1) & Dpgeo<xsup(1) & sigmageo>xinf(2) & sigmageo<xsup(2)  then
    printf('Lognormal non Overlapping spheres Fit (equation 4) : Geometric Diameter = %f et geometric standard deviation = %f\n',Dpgeo,sigmageo)
    [SSpheresfit]=SSpheres(Dfit,Dpgeo,sigmageo);
    plot2d(Dfit,SSpheresfit,5,logflag='ln');
    legend(['Non overlapping Spheres']);
    Activelegend(4)=%t;
end

legendlist=['EDM-SBS','Soot','DLCA aggregates','Non overlapping Spheres'];
legend(legendlist(Activelegend));
PlotFormatting();
xs2jpg(gcf(), 'plotEDM-SBS.jpg'); // Export to a JPG file.


//*************************************************************************************//
///////// Determination of the fractal dimension and prefactor///////////////////////////
//*************************************************************************************//
Ap=%pi/4*DmodDLCA.^2;
Np=ka*(AaTab/Ap).^alpha;
Select=(Np>5);
if sum(Select*1)>10 then
    X=2*RgTab(Select)/DmodDLCA;
    Np=Np(Select);
    p0=[1.8,1]; 
    xinf = [1, 0.1]; xsup = [3,5];
    //Fit of the fractal curve
    [VDf,p]=leastsq(FractalSearch,"b",xinf,xsup,p0);
    Df=p(1);
    kf=p(2);
    if factorDf~=0 then
        kf=1/factorDf*((Df+2)/Df).^(Df/2);
    end
    printf('Df = %f et kf = %f\n',Df,kf)
    h1=figure(1);
    clf
    set(h1,"background",8) ;//donne du blanc
    plot2d(X,Np,-5,logflag='ll')
    plot2d(X,kf*X.^Df,-2,logflag='ll'); //Courbe d'ouverture globale moyennée
    xtitle('','Rg/Rp','Np');
    PlotFormatting();
    xs2jpg(gcf(), 'plotEDM-SBS.jpg'); // Export to a JPG file.
    xs2gif(h0,'test.gif')
else 
    printf('Lack of statistique or too small aggregate size does not allow to determine the fractal properties\n')
end



